#include "qgscontexthelp.h"
#include <QCoreApplication>

QHash<QString, QString> QgsContextHelp::gContextHelpTexts;

void QgsContextHelp::init()
{
  if( !gContextHelpTexts.isEmpty() )
    return;

  gContextHelpTexts.insert( "CMakeFiles", QCoreApplication::translate( "context_help", "") );
  gContextHelpTexts.insert( "HeatmapGui", QCoreApplication::translate( "context_help", "<h3>Heatmap Plugin Help</h3>\n"
"<p>The Heatmap plugin uses Kernel Density Estimation to create a density (heatmap)\n"
"raster of an input point vector layer.  The density is calculated based on the\n"
"number of points in a location, with larger numbers of clustered points resulting\n"
"in larger values. Heatmaps allow easy identification of \"hotspots\" and\n"
"clustering of points.</p>\n"
"\n"
"<h2>Dialog Parameters</h2>\n"
"\n"
"<h3>Input point layer</h3>\n"
"<p>The input is always a vector layer of point type. All the point vector layers that are currently loaded in the canvas are shown in the drop-down list. Click the dropdown button and select the desired layer.</p>\n"
"\n"
"<h3>Output raster</h3>\n"
"<p>The output raster location and filename can be set by clicking the <label>...</label> button next to the output raster textbox.<br/>\n"
"<b>Note:</b> The file format is automatically added depending upon the output format selected, if not explicitly given.</p>\n"
"\n"
"<h3>Output format</h3>\n"
"<p>All the file creation supporting GDAL formats are available in the drop down list. Click and select the required output format for your file.<br/>\n"
"<b>Note:</b> GeoTiff and ERDAS Imagine .img formats are recommended. Some formats make the application crash. Kindly stick to the recommended formats until the crash issue is resolved or use other formats if you know GDAL supports it completely.</p>\n"
"\n"
"<h3>Radius (bandwidth)</h3>\n"
"<p>Used to specify the heatmap search radius (or kernel bandwidth) in meters or map units. The radius specifies the distance around a point at which the influence\n"
"  of the point will be felt. Larger values result in greater smoothing, but smaller values may show finer details and variation in point density.</p>\n"
"\n"
"<h3>Advanced Options</h3>\n"
"\n"
"<h4>Rows and Columns</h4>\n"
"<p>Used to change the dimensions of the output raster file. These values are also linked to the <b>Cell size X</b> and <b>Cell size Y</b> values. \n"
"Increasing the number of rows or colums will decrease the cell size and increase the file size of the output file. The values in Rows and Columns\n"
"are also linked, so doubling the number of rows will automatically double the number of columns and the cell sizes will also be halved. The geographical area of the output raster will remain the same!</p>\n"
"\n"
"<h4>Cell size X and Y</h4>\n"
"<p>Control the geographic size of each pixel in the output raster. Changing these values will also change the number of Rows and Columns in the output\n"
"  raster.</p>\n"
"  \n"
"<h4>Kernel shape</h4>\n"
"<p>The kernel shape controls the rate at which the influence of a point decreases as the distance from the point increases. Different kernels decay at\n"
"different rates, so a triweight kernel gives features greater weight for distances closer to the point than the Epanechnikov kernel does. Consequently,\n"
"triweight results in \"sharper\" hotspots, and Epanechnikov results in \"smoother\" hotspots. A number of standard kernel functions are available in QGIS,\n"
"which are described and illustrated on <a href=\"http://en.wikipedia.org/wiki/Kernel_(statistics)#Kernel_functions_in_common_use\">Wikipedia</a>.</p>\n"
"\n"
"<h4>Decay ratio</h4>\n"
"<p>For triangular kernels, the decay ratio defines amount of influence that applies to the outermost pixels in the buffer radius. A value of 0 reflects a standard triangular kernel, where the influences drops to zero as the distance from the pixel to the point approaches the specified radius. In contrast, a value of 1 indicates that no falloff occurs as the distance from the point increases.</p>\n"
"\n"
"<h4>Use radius from field</h4>\n"
"<p>Optionally, the value in a specified field can be used to control the kernel radius for each point in the input layer.</p>\n"
"\n"
"<h4>Use weight from field</h4>\n"
"<p>Optionally, a attribute containing weights for points can be used to weight some points in the input layer more than others.</p>\n"
"\n"
"<h2>Further Details</h2>\n"
"<p>Contact the author through aruntheguy at gmail dot com</p>\n"
"\n"
"") );
  gContextHelpTexts.insert( "PythonConsole", QCoreApplication::translate( "context_help", "<h3>Python Console for QGIS</h3>\n"
"<a href=\"#console\">Console</a><br>\n"
"<a href=\"#editor\">Editor</a><br>\n"
"<a href=\"#settings\">Settings</a><br><br>\n"
"<table>\n"
"<tr>\n"
"<td>\n"
"The QGIS Python Console is an interactive shell for the python command executions. \n"
"It also has a python file editor that allows you to edit and save your python scripts. \n"
"Both console and editor are based on PyQScintilla2 package.<br>\n"
"The console is split in two main panes, top and bottom one \n"
"resizable by using the horizontal splitter. Output area pane is a widget read-only which \n"
"shows the commands output. You can drag and drop or copy and paste text into input area and \n"
"execute code snippets from the output pane by selecting some text and clicking on the \n"
"<label>Enter selected</label> command from the context menu. \n"
"No matter if selected text contains the interpreter prompt (>>>, ...). Input area pane is the interactive \n"
"python shell for input commands.<br>To access to the python file editor use the \n"
"<label>Show editor</label> button \n"
"from the toolbar. The editor allows editing and saving python files and it offers basic functionality \n"
"for managing your code (comment and \n"
"uncomment code, check syntax, share the code via codepad.org and much more).\n"
"</p>\n"
"</td>\n"
"</tr>\n"
"</table>\n"
"<a name=\"console\">\n"
"<h4>Console</h4>\n"
"</a>\n"
"<b><i>Main features:</i></b>\n"
"<table>\n"
"<tr>\n"
"<td>\n"
"<ul>\n"
"<li>Code completion, highlighting syntax and calltips for the following APIs:\n"
"<ol>\n"
"<li>Python</li>\n"
"<li>PyQGIS</li>\n"
"<li>PyQt4</li>\n"
"<li>QScintilla2</li>\n"
"<li>osgeo-gdal-ogr</li>\n"
"</ol>\n"
"</li>\n"
"<br>\n"
"<li><label>Ctrl+Alt+Space</label> to view the auto-completion list.</li>\n"
"<br>\n"
"<li><label>Ctrl+Shift+Space</label> to view the command history list.</li>\n"
"<br>\n"
"<li>Execute code snippets with the <label>Enter selected</label> command from output pane.</li>\n"
"<br>\n"
"<li>Open QGIS API documentation by typing <label>_api</label>.</li>\n"
"<br>\n"
"<li>Open PyQGIS Cookbook by typing <label>_pyqgis</label>.</li>\n"
"<br>\n"
"<li>Save and clear the command history accessing from context menu of input pane. \n"
"The history will be saved into the file ~/.qgis2/console_history.txt</li>\n"
"<br>\n"
"</ul>\n"
"</td>\n"
"<td><img src=\"qrc:/images/themes/default/console/consoleHelp.png\" /></td>\n"
"</tr>\n"
"<table>\n"
"<b><i>Toolbar:</i></b>\n"
"<table width=\"100%\" id='toolbarConsole'>\n"
"<tr>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconClearConsole.png\" /></td>\n"
"<td colspan=\"2\">Clear python console</td>\n"
"</tr>\n"
"<tr>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconClassConsole.png\" /></td>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconProcessingConsole.png\" /></td>\n"
"<td>Import Processing class</td>\n"
"</tr>\n"
"<tr>\n"
"<td></td>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconQtCoreConsole.png\" /></td>\n"
"<td>Import PyQt4.QtCore class</td>\n"
"</tr>\n"
"<tr>\n"
"<td></td>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconQtGuiConsole.png\" /></td>\n"
"<td>Tool to import PyQt4.QtGui class</td>\n"
"</tr>\n"
"<tr>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconRunConsole.png\" /></td>\n"
"<td colspan=\"2\">Run command (like Enter key pressed)</td>\n"
"</tr>\n"
"<tr>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconSettingsConsole.png\" /></td>\n"
"<td colspan=\"2\">Settings</td>\n"
"</tr>\n"
"<tr>\n"
"<td><img src=\"qrc:/images/themes/default/console/iconHelpConsole.png\" /></td>\n"
"<td colspan=\"2\">Help</td>\n"
"</tr>\n"
"</table>\n"
"<a name=\"editor\">\n"
"<h4>Editor</h4>\n"
"</a>\n"
"<b><i>Main features:</i></b>\n"
"<table>\n"
"<tr>\n"
"<td>\n"
"<img src=\"qrc:/images/themes/default/console/editorHelp.png\" />\n"
"<img src=\"qrc:/images/themes/default/console/classBrowserHelp.png\" />\n"
"</td>\n"
"<td>\n"
"<ul>\n"
"<li>Code completion, highlighting syntax and calltips for the following APIs:\n"
"<ol>\n"
"<li>Python</li>\n"
"<li>PyQGIS</li>\n"
"<li>PyQt4</li>\n"
"<li>QScintilla2</li>\n"
"<li>osgeo-gdal-ogr</li>\n"
"</ol>\n"
"</li>\n"
"<br>\n"
"<li><label>Ctrl+Space</label> to view the auto-completion list.</li>\n"
"<br>\n"
"<li>Sharing code snippets via codepad.org.</li>\n"
"<br>\n"
"<li><label>Ctrl+4</label> Syntax check.</li> \n"
"<br>\n"
"<li>Object inspector: a class and function browser.</li>\n"
"<br>\n"
"<li>Go to an object definition with a mouse click. (from Object inspector)</li>\n"
"<br>\n"
"<li>Execute code snippets with the <label>Enter selected</label> command.</li>\n"
"<br>\n"
"<li>Execute the whole script with the <label>Run script</label> command \n"
"(this creates a byte-compiled file with the extension .pyc)</li>\n"
"<br>\n"
"</ul>\n"
"</td>\n"
"</tr>\n"
"</table>\n"
"<a name=\"settings\">\n"
"<h4>Settings</h4>\n"
"</a>\n"
"<b><i>Further settings for python console:</i></b>\n"
"<ul>\n"
"<li><label>Autocompletion:</label> If checked the code completion is enabled. You can get autocompletion\n"
"from current document, from installed APIs and both from APIs and current document</li>\n"
"<br>\n"
"<li><label>Autocompletion threshold:</label> Sets the threshold to display the autocompletion list (in chars typed)</li>\n"
"<br>\n"
"<li><label>Automatic parentheses insertion:</label> If checked enables the autoclosing for bracket</li>\n"
"<br>\n"
"<li><label>Auto-save script before running:</label> Allows you to save automatically the\n"
"script to be executed in order to avoid to save it after any modification.\n"
"This action will store a temporary file into the temporary system directory \n"
"that will be automatically deleted after running.</li>\n"
"<br>\n"
"<li><label>Using preloaded APIs file:</label> You can choose whether use the preload APIs file or load some APIs files saved on your system.</li>\n"
"<br>\n"
"<li><label>Using prepared APIs file:</label> If checked the *.pap file will be used for code completion. To generate a prepared APIs file you have to load least an *.api file and then compile it by clicking on <label>Compile Apis...</label> button.</li>\n"
"</ul>\n"
"<p style='border: 1px solid; background:#f6f6f6;' align='justify'>\n"
"<b>Note:</b> To save the state of console's widgets you have to close the Python Console \n"
"from the close button. This allows you to save the geometry to be restored to the next start.\n"
"</p>\n"
"") );
  gContextHelpTexts.insert( "QgsAttributeTableDialog", QCoreApplication::translate( "context_help", "<h3>Attribute table</h3>\n"
"The attribute table displays features of a selected layer. Each row in the table represents one map feature with its attributes shown in several columns. The features in the table can be searched, selected, moved or even edited.  By default the attribute table is a separate window. If you opened it and cannot see it anymore, it is most likely hidden under the main QGIS window. You can make it also a docked window, by checking <label>Open attribute table in a dock window</label> in <label>Settings > Options > Data Sources</label>.<p>\n"
"\n"
"The number of total number of layer features, filtered features and selected features is printed on the window header.<p>\n"
"At the bottom right of the dialog, you can choose whether to display features in a form or in a table.<p>\n"
"<p>\n"
"<a href=\"#Selecting\">Selecting</a><br/>\n"
"<a href=\"#Sorting\">Sorting</a><br/>\n"
"<a href=\"#Filtering\">Filtering</a><br/>\n"
"<a href=\"#Editing\">Editing</a><br/>\n"
"<a href=\"#FieldCalc\">Field Calculator</a><br/>\n"
"<a href=\"#FieldCalcBar\">Field Calculator bar</a><br/>\n"
"\n"
"<a name=\"Selecting\">\n"
"<h4>Selecting</h4>\n"
"</a>\n"
"Rows can be selected by clicking on the row number on the left side of the row. Subsequent rows can be selected by holding the mouse button down and moving the mouse to the other end of the selection. Multiple non-subsequent rows can be selected by holding the <label>Ctrl</label> key.<br>\n"
"A continuous selection can be made by holding the <label>Shift</label> key and clicking on several row headers on the left side of the rows. All rows between the current cursor position and the clicked row are selected.\n"
"\n"
"<a name=\"Sorting\">\n"
"<h4>Sorting</h4>\n"
"</a>\n"
"Each column can be sorted by clicking on its column header. A small arrow indicates the sort order (downward pointing means descending values from the top row down, upward pointing means ascending values from the top row down).<br>\n"
"\n"
"<a name=\"Filtering\">\n"
"<h4>Filtering</h4>\n"
"</a>\n"
"To only navigate through parts of your data, you can use the filter button on the bottom left. The following options are available.\n"
"<h5>Show All Features</h5>\n"
"Shows all features of the layer.\n"
"<h5>Show Selected Features</h5>\n"
"Shows all features which are currently selected.\n"
"<h5>Show Features Visible on Map</h5>\n"
"Shows all features which are currently visible on the map canvas, taking the currently visible extent and scale based visibility into account.\n"
"<h5>Show Edited and New Features</h5>\n"
"Shows only edited and new features. In this mode, features with uncommitted changes are shown and this is therefore a good filter to review changes, before committing them. Please note, that deleted features are not shown in this mode.\n"
"<h5>Column Filter</h5>\n"
"A simple filter, which lets you filter by an attribute. If the attribute contains text, it looks for sub-strings also. Searching for <b>man</b> will therefore also show records containing <b>woman</b>. After changing the filter text, press <label>Enter</label> or click on <label>Apply</label>. You can also toggle the check-box <label>Case sensitive</label>, so your filter will also match <b>Night</b> when your search text is <b>night</b>.   \n"
"<h5>Advanced Filter</h5>\n"
"For more complex searches, this mode offers a powerful expression builder, which is similar to a SQL WHERE clause. Please refer to the built-in help of the expression builder for details on the syntax.\n"
"\n"
"<a name=\"Editing\">\n"
"<h4>Editing</h4>\n"
"</a>\n"
"To edit values, you first have to switch the layer to editing mode. To switch to editing mode click on the <label>Toggle editing mode</label> (pencil) button or press <label>Ctrl + E</label>. Afterwards double-click on the value you want to edit or place the cursor on it and use the <label>Space</label>-Key. You can customize the widgets used for field editing in <label>Vector Layer Properties > Fields</label>  <br>\n"
"You can also edit a field's value for many rows at once. You need to select the rows you want to edit and then use the <label>Field Calculator</label> or the <label>Field Calculator Bar</label>. If no row is selected, the edition will apply on all the features.\n"
"\n"
"<a name=\"FieldCalc\">\n"
"<h4>Field Calculator</h4>\n"
"</a>\n"
"The <label>Field Calculator</label> button in the attribute table allows performing calculations on basis of existing attribute values or defined functions, e.g. to calculate length or area of geometry features.<br>\n"
"The results can be written to a new attribute column or it can be used to update values in an already existing column.<br>\n"
"The vector layer needs to be in editing mode, before you can click on the <label>Field calculator</label> icon to open the dialog.\n"
"\n"
"<a name=\"FieldCalcBar\">\n"
"<h4>Field Calculator Bar</h4>\n"
"</a>\n"
"When you toggle to edition the layer, a new row of functions appears in the attribute table: the <label>Field Calculator Bar</label>. This allows to quickly edit values of existing fields by performing calculations on basis of existing attribute values or defined functions button in the attribute table, e.g. to calculate length or area of geometry features.<br>\n"
"To edit values, select the field to modify with the filter button on the left and fill the text box with the new value or an expression to calculate new value. Then, press <label>Update all</label> button to update all the rows of the attribute table or <label>Update selected</label> button if some features are selected or a filter is applied on the attribute table display. You can also use the <label>Expression builder</label> button to calculate the new value in the column.<br>\n"
"") );
  gContextHelpTexts.insert( "QgsBookmarks", QCoreApplication::translate( "context_help", "<h3>Spatial Bookmarks</h3>\n"
"Spatial Bookmarks allow you to \"bookmark\" a geographic location and return to it later.\n"
"<p>\n"
"<a href=\"#creating\">Creating a Bookmark</a><br/>\n"
"<a href=\"#working\">Working with Bookmarks</a><br/>\n"
"<a href=\"#zooming\">Zooming to a Bookmark</a><br/>\n"
"<a href=\"#deleting\">Deleting a Bookmark</a><br/>\n"
"<a href=\"#updating\">Updating a Bookmark</a><br/>\n"
"\n"
"<a name=\"creating\">\n"
"<h4>Creating a Bookmark</h4>\n"
"</a>\n"
"To create a bookmark:\n"
"<ol>\n"
"<li>Zoom or pan to the area of interest.\n"
"<li>Click on the <label>New bookmark</label> or type Ctrl-B.\n"
"<li>Enter a descriptive name for the bookmark (up to 255 characters).\n"
"<li>Click <label>OK</label> to add the bookmark or <label>Cancel</label> to exit without adding the bookmark.\n"
"</ol>\n"
"Note that you can have multiple bookmarks with the same name.\n"
"<a name=\"working\">\n"
"<h4>Working with Bookmarks</h4>\n"
"</a>\n"
"To use or manage bookmarks, click on the <label>Show bookmarks</label> button or choose <label>Show Bookmarks</label> from the <label>View</label> menu. The bookmarks dialog allows you to zoom to or delete a bookmark. You can not edit the bookmark name or coordinates.\n"
"<a name=\"zooming\">\n"
"<h5>Zooming to a Bookmark</h5>\n"
"</a>\n"
"From the Geo spatial Bookmarks dialog, select the desired bookmark by clicking on it, then click the <label>Zoom to</label> button.\n"
"<p>\n"
"You can also zoom to a bookmark by double-clicking on it.\n"
"<a name=\"deleting\">\n"
"<h5>Deleting a Bookmark</h5>\n"
"</a>\n"
"To delete a bookmark from the Bookmarks dialog, click on it then click the <label>Delete</label> button. Confirm your choice by clicking <label>OK</label> or cancel the delete by clicking <label>Cancel</label>.\n"
"<a name=\"updating\">\n"
"<h5>Updating a Bookmark</h5>\n"
"</a>\n"
"To update the extent of a bookmark, click on it then click the <label>Update</label> button. Confirm your choice by clicking <label>OK</label> or cancel the update by clicking <label>Cancel</label>.\n"
"\n"
"") );
  gContextHelpTexts.insert( "QgsComposer", QCoreApplication::translate( "context_help", "<h3>Print Composer</h3>\n"
"\n"
"<p>\n"
"The print composer lets you arrange items onto a page, which can then be printed. Items include the content of the map canvas, the scale bar, the legend, and arbitrary images.\n"
"</p>\n"
"\n"
"<p>\n"
"Click on one of the icons along the top of the composer window to select what type of item to place on the page, then click and draw on the page to place the item. The size of some items, such as the map canvas and scale bar can be set by clicking and dragging on the page. Other items have a fixed size, but which can be changed later on by highlighting the item and then adjusting the size fields in the left hand pane.\n"
"\n"
"</p>\n"
"\n"
"") );
  gContextHelpTexts.insert( "QgsCustomProjectionDialog", QCoreApplication::translate( "context_help", "<h3>Custom Coordinate Reference System Definition</h3>\n"
"If QGIS does not provide the coordinate reference system you need, you can define a custom CRS. <p>\n"
"To define a CRS, select <label>Custom CRS</label> from the <label>Edit</label> (Gnome, OSX) or <label>Settings</label> (KDE, Windows) menu. Custom CRS are stored in your QGIS user database. In addition to your custom CRS, this database also contains your spatial bookmarks and other custom data.\n"
"\n"
"<h4>Define</h4>\n"
"To create a new CRS, click the <label>New</label> button and enter:\n"
"<ul>\n"
"<li>descriptive name \n"
"<li>CRS parameters\n"
"</ul>\n"
"Note that the Parameters must begin with a +proj=-block, to represent the new coordinate reference\n"
"system.\n"
"<p>\n"
"Defining a custom CRS in QGIS requires a good understanding of the Proj.4 projection library. To begin, refer to the <a href=\"ftp://ftp.remotesensing.org/proj/OF90-284.pdf\">Cartographic Projection Procedures for the UNIX Environment</a> - A User's Manual by Gerald I. Evenden, U.S. Geological Survey Open-File Report 90-284, 1990. This manual describes the use of the <i>proj.4</i> and related command line utilities. The cartographic parameters used with <i>proj.4</i> are described in the user manual, and are the same as those used by QGIS.\n"
"") );
  gContextHelpTexts.insert( "QgsDecorationCopyrightDialog", QCoreApplication::translate( "context_help", "<h3>Copyright Label</h3>\n"
"Draws a copyright label with information.<br/>\n"
"You can add any random text to the map not only a copyright label.\n"
"<p>\n"
"<a href=\"#howto\">How to</a><br/>\n"
"<a href=\"#text\">Text formatting</a><br/>\n"
"\n"
"<a name=\"howto\">\n"
"<h4>How to</h4>\n"
"</a>\n"
"<ol>\n"
"<li>Make sure the plugin is loaded.\n"
"<li>Click on <label>View > Decorations > Copyright Label</label>.\n"
"<li>Enter the text you want to place on the map. You can use HTML.\n"
"<li>Choose the placement of the label from the <label>Placement Bottom Right</label> drop-down box.\n"
"<li>Make sure the <label>Enable Copyright Label check box</label> is checked.\n"
"<li>Click <label>OK</label>.\n"
"</ol>\n"
"<a name=\"text\">\n"
"<h4> Text formatting </h4>\n"
"</a>\n"
"This plugin supports basic HTML markup tags for formatting text. For example:\n"
"\n"
"<ul>\n"
"<li>&lt;B&gt;<b> Bold text </b>&lt;/B&gt;\n"
"</li>\n"
"<li>&lt;I&gt;<i> Italics </i>&lt;/I&gt;\n"
"</li>\n"
"<li>&lt;U&gt;<u> Underlined text </u>&lt;/U&gt;\n"
"</li>\n"
"</ul>\n"
"") );
  gContextHelpTexts.insert( "QgsDecorationNorthArrowDialog", QCoreApplication::translate( "context_help", "<h3> North Arrow Plugin</h3>\n"
"Displays a north arrow overlayed onto the map.\n"
"<p>\n"
"At present there is only one style available. You can adjust the angle of the arrow or let QGIS set the direction automatically. If you choose to let QGIS determine the direction, it makes its best guess as to how the arrow should be oriented. For placement of the arrow you have four options, corresponding to the four corners of the map canvas.") );
  gContextHelpTexts.insert( "QgsDecorationScaleBarDialog", QCoreApplication::translate( "context_help", "<h3>Scale bar Plugin</h3>\n"
"Draws a scale bar on the map.\n"
"<p>\n"
"You can control the style and placement, as well as the labeling of the bar.\n"
"QGIS only supports displaying the scale in the same units as your map frame. So\n"
"if the units of your layers are in meters, you can't create a scale bar in\n"
"feet. Likewise if you are using decimal degrees, you can't create a scale bar\n"
"to display distance in meters.</p> <p>\n"
"To add a scale bar:\n"
"<ol>\n"
"<li>Choose the placement from the <label>Placement</label> drop-down list.</li>\n"
"<li>Choose the style from the <label>Scale bar style</label> list.</li>\n"
"<li>Select the color for the bar <label>Color of bar</label> or use the default black color.</li>\n"
"<li>Set the size of the bar and its label <label>Size of bar</label>. </li>\n"
"<li>Make sure the <label>Enable scale bar</label> checkbox is checked.</li>\n"
"<li>Optionally choose to automatically snap to a round number when the canvas is resized <label>Automatically snap to round number on resize</label>.</li>\n"
"<li>Click <label>OK</label>.</li>\n"
"</ol>\n"
"") );
  gContextHelpTexts.insert( "QgsDelimitedTextSourceSelect", QCoreApplication::translate( "context_help", "<h3>Delimited Text File Layer</h3>\n"
"Loads and displays delimited text files\n"
"<p>\n"
"<a href=\"#re\">Overview</a><br/>\n"
"<a href=\"#creating\">Creating a delimited text layer</a><br/>\n"
"<a href=\"#csv\">How the delimiter, quote, and escape characters work</a><br />\n"
"<a href=\"#regexp\">How regular expression delimiters work</a><br />\n"
"<a href=\"#wkt\">How WKT text is interpreted</a><br />\n"
"<a href=\"#attributes\">Attributes in delimited text files</a><br />\n"
"<a href=\"#example\">Example of a text file with X,Y point coordinates</a><br/>\n"
"<a href=\"#wkt_example\">Example of a text file with WKT geometries</a><br/>\n"
"<a href=\"#python\">Using delimited text layers in Python</a><br/>\n"
"</p>\n"
"\n"
"<h4><a name=\"re\">Overview</a></h4>\n"
"<p>A &quot;delimited text file&quot; contains data in which each record starts on a new line, and \n"
"is split into fields by a delimiter such as a comma.  \n"
"This type of file is commonly exported from spreadsheets (for example CSV files) or databases.  \n"
"Typically the first line of a delimited text file contains the names of the fields.  \n"
"</p>\n"
"<p>\n"
"Delimited text files can be loaded into QGIS as a layer. \n"
"The records can be displayed spatially either as a point\n"
"defined by X and Y coordinates, or using a Well Known Text (WKT) definition of a geometry which may\n"
"describe points, lines, and polygons of arbitrary complexity.  The file can also be loaded as an attribute\n"
"only table, which can then be joined to other tables in QGIS.\n"
"</p>\n"
"<p>\n"
"In addition to the geometry definition the file can contain text, integer, and real number fields.  By default \n"
"QGIS will choose the type of field based on its the non blank values of the field.  If all can be interpreted\n"
"as integer then the type will be integer, if all can be interpreted as real numbers then the type will\n"
"be double, otherwise the type will be text.\n"
"</p>\n"
"<p>\n"
"QGIS can also read the types from an OGR CSV driver compatible &quot;csvt&quot; file.  \n"
"This is a file alongside the data file, but with a &quot;t&quot; appended to the file name. \n"
"The file should just contain one line which lists the type of each field. \n"
"Valid types are &quot;integer&quot;, &quot;real&quot;, &quot;string&quot;, &quot;date&quot;, &quot;time&quot;, and &quot;datetime&quot;. The date, time, and datetime types are treated as strings in QGIS.\n"
"Each type may be followed by a width and precision, for example &quot;real(10.4)&quot;.\n"
"The list of types are separated by commas, regardless of the delimiter used in the data file.  An\n"
"example of a valid format file would be:\n"
"</p>\n"
"\n"
"<pre>\n"
"&quot;integer&quot;,&quot;string&quot;,&quot;string(20)&quot;,&quot;real(20.4)&quot;\n"
"</pre>\n"
"\n"
"<h4><a name=\"creating\">Creating a delimited text layer</a></h4>\n"
"<p>Creating a delimited text layer involves choosing the data file, defining the format (how each record is to\n"
"be split into fields), and defining the geometry is represented.  \n"
"This is managed with the delimited text dialog as detailed below.  \n"
"The dialog box displays a sample from the beginning of the file which shows how the format\n"
"options have been applied.\n"
"</p>\n"
"<h5>Choosing the data file</h5>\n"
"<p>Use the &quot;Browse...&quot; button to select the data file.  Once the file is selected the\n"
"layer name will automatically be populated based on the file name.  The layer name is used to represent\n"
"the data in the QGIS legend.  \n"
"</p>\n"
"<p>\n"
"By default files are assumed to be encoded as UTF-8.  However other file\n"
"encodings can be selected.  For example &quot;System&quot; uses the default encoding for the operating system.  \n"
"It is safer to use an explicit coding if the QGIS project needs to be portable.\n"
"</p>\n"
"<h5>Specifying the file format</h5>\n"
"<p>The file format can be one of\n"
"<ul>\n"
"    <li>CSV file format.  This is a format commonly used by spreadsheets, in which fields are delimited\n"
"    by a comma character, and quoted using a &quot;(quote) character.  Within quoted fields, a quote\n"
"    mark is entered as &quot;&quot;.</li>\n"
"    <li>Selected delimiters.  Each record is split into fields using one or more  delimiter character.\n"
"    Quote characters are used for fields which may contain delimiters.  Escape characters may be used \n"
"    to treat the following character as a normal character (ie to include delimiter, quote, and \n"
"    new line characters in text fields).  The use of delimiter, quote, and escape characters is detailed <a href=\"#csv\">below</a>.\n"
"    <li>Regular expression.  Each line is split into fields using a &quot;regular expression&quot; delimiter.\n"
"    The use of regular expressions is details <a href=\"#regexp\">below</a>.\n"
"</ul>\n"
"<h5>Record and field options</h5>\n"
"<p>The following options affect the selection of records and fields from the data file</p>\n"
"<ul>\n"
"    <li>Number of header lines to discard: used to ignore header lines at the beginning of the text file</li>\n"
"    <li>First record has fields names: if selected then the first record in the file (after the discarded lines) is interpreted as names of fields, rather than as a data record.</li>\n"
"    <li>Trim fields: if selected then leading and trailing whitespace characters will be removed from each field (except quoted fields). </li>\n"
"    <li>Discard empty fields: if selected then empty fields (after trimming) will be discard.  This \n"
"    affects the alignment of data into fields and is equivalent to treating consecutive delimiters as a \n"
"    single delimiter.  Quoted fields are never discarded.</li>\n"
"    <li>Decimal separator is comma: if selected then commas instead of points are used as the decimal separator in real numbers.  For\n"
"    example <tt>-51,354</tt> is equivalent to -51.354.\n"
"    </li>\n"
"</ul>\n"
"<h5>Geometry definition</h5>\n"
"<p>The geometry is can be define as one of</p>\n"
"<ul>\n"
"    <li>Point coordinates: each feature is represented as a point defined by X and Y coordinates.</li>\n"
"    <li>Well known text (WKT) geometry: each feature is represented as a well known text string, for example\n"
"    <tt>POINT(1.525622 51.20836)</tt>.  See details of the <a href=\"#wkt\">well known text</a> format.\n"
"    <li>No geometry (attribute only table): records will not be displayed on the map, but can be viewed\n"
"    in the attribute table and joined to other layers in QGIS</li>\n"
"</ul>\n"
"<p>For point coordinates the following options apply:</p>\n"
"<ul>\n"
"    <li>X field: specifies the field containing the X coordinate</li>\n"
"    <li>Y field: specifies the field containing the Y coordinate</li>\n"
"    <li>DMS angles: if selected coordinates are represented as degrees/minutes/seconds\n"
"    or degrees/minutes.  QGIS is quite permissive in its interpretation of degrees/minutes/seconds.\n"
"    A valid DMS coordinate will contain three numeric fields with an optional hemisphere prefix or suffix\n"
"    (N, E, or + are positive, S, W, or - are negative).  Additional non numeric characters are \n"
"    generally discarded.  For example <tt>N41d54'01.54&quot;</tt> is a valid coordinate.\n"
"    </li>\n"
"</ul>\n"
"<p>For well known text geometry the following options apply:</p>\n"
"<ul>\n"
"    <li>Geometry field: the field containing the well known text definition.</li>\n"
"    <li>Geometry type: one of &quot;Detect&quot; (detect), &quot;Point&quot;, &quot;Line&quot;, or &quot;Polygon&quot;.\n"
"    QGIS layers can only display one type of geometry feature (point, line, or polygon). This option selects\n"
"    which geometry type is displayed in text files containing multiple geometry types. Records containing\n"
"   other geometry types are discarded.   \n"
"    If &quot;Detect&quot; is selected then the type of the first geometry in the file will be used.\n"
"    &quot;Point&quot; includes POINT and MULTIPOINT WKT types, &quot;Line&quot; includes LINESTRING and\n"
"    MULTLINESTRING WKT types, and &quot;Polygon&quot; includes POLYGON and MULTIPOLYGON WKT types.\n"
"</ul>\n"
"<h5>Layer settings</h5>\n"
"<p>Layer settings control the way the layer is managed in QGIS.  The options available are:</p>\n"
"<ul>\n"
"<li>Use spatial index. Create a spatial index to improve the performance of displaying and selecting spatial objects.\n"
"This option may be useful for files larger than a few megabytes in size.</li>\n"
"<li>Use subset index. Create an index if a subset of records is being used (either by explicitly setting a subset string \n"
"from the layer properties dialog, or an implicit subset of features for which the geometry is valid in files\n"
"for which all not geometries are valid).  The index will only be created when a subset is defined.</li>\n"
"<li>Watch file.  If this options is selected QGIS will watch the file for changes by other applications, and \n"
"reload the file when it is changed.  The map will not be updated until refreshed by the user, but indexes and\n"
"extents will be reloaded.  This option should be selected if indexes are used and it is likely that another\n"
"application will change the file. </li>\n"
"</ul>\n"
"\n"
"<h4><a name=\"csv\">How the delimiter, quote, and escape characters work</a></h4>\n"
"<p>Records are split into fields using three character sets: \n"
"delimiter characters, quote characters, and escape characters.  \n"
"Other characters in the record are considered as data, split into\n"
"fields by delimiter characters. \n"
"Quote characters occur in pairs and cause the text between them to be treated as a data.  Escape characters cause the character following them to be treated as data.   \n"
"</p>\n"
"<p>\n"
"Quote and escape characters cannot be the same as delimiter characters - they\n"
"will be ignored if they are. Escape characters can be the same as quote characters, but behave differently\n"
"if they are.</p>\n"
"<p>The delimiter characters are used to mark the end of each field.  If more than one delimiter character\n"
"is defined then any one of the characters can mark the end of a field.  The quote and escape characters \n"
"can override the delimiter character, so that it is treated as a normal data character.</p>\n"
"<p>Quote characters may be used to mark the beginning and end of quoted fields. Quoted fields can \n"
"contain delimiters and may span multiple lines in the text file.  If a field is quoted then it must\n"
"start and end with the same quote character.  Quote characters cannot occur within a field unless they\n"
"are escaped.</p>\n"
"<p>Escape characters which are not quote characters force the following character to be treated as data.  \n"
"(that is, to stop it being treated as a new line, delimiter, or quote character).  \n"
"</p>\n"
"<p>Escape characters that are also quote characters have much more limited effect.  They only apply within quotes and only escape themselves.  For example, if \n"
"<tt>'</tt> is a quote and escape character, then the string\n"
"<tt>'Smith''s&nbsp;Creek'</tt> will represent the value Smith's&nbsp;Creek.\n"
"</p>\n"
"\n"
"\n"
"<h4><a name=\"regexp\">How regular expression delimiters work</a></h4>\n"
"<p>Regular expressions are mini-language used to represent character patterns.  There are many variations\n"
"of regular expression syntax - QGIS uses the syntax provided by the <a href=\"http://qt-project.org/doc/qt-4.8/qregexp.html\">QRegExp</a> class of the <a href=\"http://qt.digia.com\">Qt</a> framework.</p>\n"
"<p>In a regular expression delimited file each line is treated as a record.  Each match of the regular expression in the line is treated as the end of a field.  \n"
"If the regular expression contains capture groups (eg <tt>(cat|dog)</tt>)\n"
" then these are extracted as fields. \n"
" If this is not desired then use non-capturing groups (eg <tt>(?:cat|dog)</tt>).\n"
"</p>\n"
"<p>The regular expression is treated differently if it is anchored to the start of the line (that is, the pattern starts with <tt>^</tt>).\n"
"In this case the regular expression is matched against each line.  If the line does not match it is discarded\n"
"as an invalid record.  Each capture group in the expression is treated as a field.  The regular expression\n"
"is invalid if it does not have capture groups.  As an example this can be used as a (somewhat \n"
"unintuitive) means of loading data with fixed width fields.  For example the \n"
"expression\n"
"<pre>\n"
"^(.{5})(.{10})(.{20})(.{20})\n"
"</pre>\n"
"<p>will extract four fields of widths 5, 10, 20, and 20 characters from each line.  \n"
"Lines less than 55 characters long will be discarded.\n"
"</p>\n"
"\n"
"\n"
"<h4><a name=\"wkt\">How WKT text is interpreted</a></h4>\n"
"<p>\n"
"The delimited text layer recognizes the following \n"
"<a href=\"http://en.wikipedia.org/wiki/Well-known_text\">well known text</a> types - \n"
"<tt>POINT</tt>, <tt>MULTIPOINT</tt>, <tt>LINESTRING</tt>, <tt>MULTILINESTRING</tt>, <tt>POLYGON</tt>, and <tt>MULTIPOLYGON</tt>.  \n"
"It will accept geometries with\n"
"a Z coordinate (eg <tt>POINT&nbsp;Z</tt>), a measure (<tt>POINT&nbsp;M</tt>), or both (<tt>POINT&nbsp;ZM</tt>).\n"
"</p>\n"
"<p>\n"
"It can also handle the PostGIS EWKT variation, in which the geometry is preceded by an spatial reference \n"
"system id (eg <tt>SRID=4326;POINT(175.3&nbsp;41.2)</tt>), and a variant used by Informix in which the WKT is \n"
"preceded by an integer spatial reference id (eg <tt>1 POINT(175.3&nbsp;41.2)</tt>).\n"
"In both cases the SRID is ignored.\n"
"</p>\n"
"\n"
"\n"
"\n"
"<h4><a name=\"attributes\">Attributes in delimited text files</a></h4> \n"
"<p>Each record in the delimited text file is split into fields representing\n"
"attributes of the record.  Usually the attribute names are taken from the first\n"
"data record in the file.  However if this does not contain attribute names, then they will be named <tt>field_1</tt>, <tt>field_2</tt>, and so on.  \n"
"Also if records have more fields than are defined in the header record then these\n"
"will be named <tt>field_#</tt>, where # is the field number (note that empty fields at the end of a record are ignored).\n"
"QGIS may override \n"
"the names in the text file if they are numbers, or have names like <tt>field_#</tt>,\n"
"or are duplicated.\n"
"</p>\n"
"<p>\n"
"In addition to the attributes explicitly in the data file QGIS assigns a unique \n"
"feature id to each record which is the line number in the source file on which\n"
"the record starts.  \n"
"</p>\n"
"<p>\n"
"Each attribute also has a data type, one of string (text), integer, or real number.\n"
"The data type is inferred from the content of the fields - if every non blank value\n"
"is a valid integer then the type is integer, otherwise if it is a valid real\n"
"number then the type is real, otherwise the type is string.  Note that this is\n"
"based on the content of the fields - quoting fields does not change the way they\n"
"are interpreted.\n"
"</p>\n"
"\n"
"\n"
"<h4><a name=\"example\">Example of a text file with X,Y point coordinates</a></h4> \n"
"<pre>\n"
"X;Y;ELEV\n"
"-300120;7689960;13\n"
"-654360;7562040;52\n"
"1640;7512840;3\n"
"</pre>\n"
"<p>This file:</p>\n"
"<ul>\n"
"<li> Uses <b>;</b> as delimiter. Any character can be used to delimit the fields.</li>\n"
"<li>The first row is the header row. It contains the field names X, Y and ELEV.</li>\n"
"<li>The x coordinates are contained in the X field.</li>\n"
"<li>The y coordinates are contained in the Y field.</li>\n"
"</ul>\n"
"<h4><a name=\"wkt_example\">Example of a text file with WKT geometries</a></h4>\n"
"<pre>\n"
"id|wkt\n"
"1|POINT(172.0702250 -43.6031036)\n"
"2|POINT(172.0702250 -43.6031036)\n"
"3|POINT(172.1543206 -43.5731302)\n"
"4|POINT(171.9282585 -43.5493308)\n"
"5|POINT(171.8827359 -43.5875983)\n"
"</pre>\n"
"<p>This file:</p>\n"
"<ul>\n"
"  <li>Has two fields defined in the header row: id and wkt.\n"
"  <li>Uses <b>|</b> as a delimiter.</li>\n"
"  <li>Specifies each point using the WKT notation\n"
"</ul>\n"
"\n"
"<h4><a name=\"python\">Using delimited text layers in Python</a></h4>\n"
"<p>Delimited text data sources can be creating from Python in a similar way to other vector layers.\n"
"The pattern is:\n"
"</p>\n"
"<pre>\n"
"from PyQt4.QtCore import QUrl, QString\n"
"from qgis.core import QgsVectorLayer, QgsMapLayerRegistry\n"
"\n"
"# Define the data source\n"
"filename=\"test.csv\"\n"
"uri=QUrl.fromLocalFile(filename)\n"
"uri.addQueryItem(\"type\",\"csv\")\n"
"uri.addQueryItem(\"delimiter\",\"|\")\n"
"uri.addQueryItem(\"wktField\",\"wkt\")\n"
"# ... other delimited text parameters\n"
"layer=QgsVectorLayer(QString(uri.toEncoded()),\"Test CSV layer\",\"delimitedtext\")\n"
"# Add the layer to the map\n"
"if layer.isValid():\n"
"    QgsMapLayerRegistry.instance().addMapLayer( layer )\n"
"</pre>\n"
"<p>This could be used to load the second example file above.</p>\n"
"<p>The configuration of the delimited text layer is defined by adding query items to the uri.\n"
"The following options can be added\n"
"</p>\n"
"<ul>\n"
"    <li><tt>encoding=..</tt> defines the file encoding.  The default is &quot;UTF-8&quot;</li>\n"
"    <li><tt>type=(csv|regexp|whitespace)</tt> defines the delimiter type.  Valid values are csv, \n"
"       regexp, and whitespace (which is just a special case of regexp).  The default is csv.</li>\n"
"       <li><tt>delimiter=...</tt> defines the delimiters that will be used for csv formatted files, \n"
"       or the regular expression for regexp formatted files.  The default is , for CSV files.  There is\n"
"       no default for regexp files.</li>\n"
"       <li><tt>quote=..</tt> (for csv files) defines the characters used to quote fields. The default is &quot;</li>\n"
"       <li><tt>escape=..</tt> (for csv files) defines the characters used to escape the special meaning of the next character. The default is &quot;</li>\n"
"       <li><tt>skipLines=#</tt> defines the number of lines to discard from the beginning of the file. The default is 0.</li>\n"
"       <li><tt>useHeader=(yes|no)</tt> defines whether the first data record contains the names of the data fields. The default is yes.</li>\n"
"       <li><tt>trimFields=(yes|no)</tt> defines whether leading and trailing whitespace is to be removed from unquoted fields. The default is no.</li>\n"
"       <li><tt>maxFields=#</tt> defines the maximum number of fields that will be loaded from the file.  \n"
"       Additional fields in each record will be discarded. The default is 0 - include all fields.\n"
"       (This option is not available from the delimited text layer dialog box).</li>\n"
"       <li><tt>skipEmptyFields=(yes|no)</tt> defines whether empty unquoted fields will be discarded (applied after trimFields). The default is no.</li>\n"
"       <li><tt>decimalPoint=.</tt> specifies an alternative character that may be used as a decimal point in numeric fields.  The default is a point (full stop) character.</li>\n"
"       <li><tt>wktField=fieldname</tt> specifies the name or number (starting at 1) of the field containing a well known text geometry definition</li>\n"
"       <li><tt>xField=fieldname</tt> specifies the name or number (starting at 1) of the field the X coordinate (only applies if wktField is not defined)</li>\n"
"       <li><tt>yField=fieldname</tt> specifies the name or number (starting at 1) of the field the Y coordinate (only applies if wktField is not defined)</li>\n"
"       <li><tt>geomType=(auto|point|line|polygon|none)</tt> specifies type of geometry for wkt fields, or none to load the file as an attribute-only table.  The default is auto.</li>\n"
"       <li><tt>subset=expression</tt> specifies an expression used to identify a subset of the records that will be \n"
"       used.</li>\n"
"       <li><tt>crs=...</tt> specifies the coordinate system to use for the vector layer, in a format accepted by QgsCoordinateReferenceSystem.createFromString (for example &quot;EPSG:4167&quot;).  If this is not \n"
"       specified then a dialog box may request this information from the user\n"
"       when the layer is loaded (depending on QGIS CRS settings).</li>\n"
"       <li><tt>subsetIndex=(yes|no)</tt> specifies whether the provider should build an index to define subset during the initial file scan.  The index will apply both for explicitly defined subsets, and for the implicit subset of features for which the geometry definition is valid.  By default the subset index is built if it is applicable.</li>\n"
"       <li><tt>spatialIndex=(yes|no)</tt> specifies whether the provider should build a spatial index during the initial file scan.  By default the spatial index is not built. </li>\n"
"       <li><tt>watchFile=(yes|no)</tt> specifies whether the provider should use a file system watcher to monitor for changes to the file.</li>\n"
"       <li><tt>quiet=(yes|no)</tt> specifies whether errors encountered loading the layer are presented in a dialog box (they will be written to the QGIS log in any case). The default is no.  This option is not available from the GUI</li>\n"
"</ul>\n"
"\n"
"\n"
"") );
  gContextHelpTexts.insert( "QgsFieldCalculator", QCoreApplication::translate( "context_help", "<h3>Field Calculator</h3>\n"
"The field calculator allows you to update fields with expressions.\n"
"\n"
"<h4>Supported Operations</h4>\n"
"\n"
"<table border=1>\n"
"<tr>\n"
"  <th>Operation</th>\n"
"  <th>Description</th>\n"
"</tr>\n"
"<tr>\n"
"  <td>\n"
"    <tt>column_name</tt><br>\n"
"    <tt>\"column_name\"</tt>\n"
"  </td>\n"
"  <td>value of field <tt>column_name</tt></td>\n"
"</tr>\n"
"<tr><td>'<tt>string</tt>'</td><td>literal string value</td></tr>\n"
"<tr><td><tt>number</tt></td><td>number</td></tr>\n"
"<tr><td>NULL</td><td>null value</td></tr>\n"
"<tr><td>exp(<tt>a</tt>)</td><td>Euler's number e to the power of <tt>a</tt></td></tr>\n"
"<tr><td>ln(<tt>a</tt>)</td><td>natural log (i.e. base e) of <tt>a</tt></td></tr>\n"
"<tr><td>log10(<tt>a</tt>)</td><td>log base 10 of <tt>a</tt></td></tr>\n"
"<tr><td>log(<tt>a</tt>,<tt>b</tt>)</td><td>log base <tt>b</tt> of <tt>a</tt></td></tr>\n"
"<tr><td>sqrt(<tt>a</tt>)</td><td>square root</td></tr>\n"
"<tr><td>sin(<tt>a</tt>)</td><td>sine of <tt>a</tt></td></tr>\n"
"<tr><td>cos(<tt>a</tt>)</td><td>cosine of <tt>b</tt></td></tr>\n"
"<tr><td>tan(<tt>a</tt>)</td><td>tangent of <tt>a</tt></td></tr>\n"
"<tr><td>asin(<tt>a</tt>)</td><td>inverse sine of <tt>a</tt></td></tr>\n"
"<tr><td>acos(<tt>a</tt>)</td><td>inverse cosine of <tt>a</tt></td></tr>\n"
"<tr><td>atan(<tt>a</tt>)</td><td>inverse tangent of <tt>a</tt></td></tr>\n"
"<tr><td>to int(<tt>a</tt>)</td><td>convert string <tt>a</tt> to integer</td></tr>\n"
"<tr><td>to real(<tt>a</tt>)</td><td>convert string <tt>a</tt> to real</td></tr>\n"
"<tr><td>to string(<tt>a</tt>)</td><td>convert number <tt>a</tt> to string</td></tr>\n"
"<tr><td>lower(<tt>a</tt>)</td><td>convert string <tt>a</tt> to lower case</td></tr>\n"
"<tr><td>upper(<tt>a</tt>)</td><td>convert string <tt>a</tt> to upper case</td></tr>\n"
"<tr><td>length(<tt>a</tt>)</td><td>length of string <tt>a</tt></td></tr>\n"
"<tr><td>atan2(<tt>y</tt>,<tt>x</tt>)</td><td>inverse tangent of <tt>y</tt>/<tt>x</tt> using the signs of the two arguments to determine the quadrant of the result.</td></tr>\n"
"<tr><td>replace(<tt>a</tt>,<tt>replacethis</tt>,<tt>withthat</tt>)</td><td>replace <tt>replacethis</tt> with <tt>withthat</tt> in string <tt>a</tt></td></td>\n"
"<tr><td>regexp_replace(<tt>a</tt>,<tt>replacethis</tt>,<tt>withthat</tt>)</td><td>replace the regular expression <tt>replacethis</tt> with <tt>withthat</tt> in string <tt>a</tt></td></td>\n"
"<tr><td>substr(<tt>a</tt>,<tt>from</tt>,<tt>len</tt>)</td><td><tt>len</tt> characters of string <tt>a</tt> starting from <tt>from</tt> (first character index is 1)</td></td>\n"
"<tr><td><tt>a</tt> || <tt>b</tt></td><td>concatenate strings <tt>a</tt> and <tt>b</tt></td></tr>\n"
"<tr><td>$rownum</td><td>number current row</td></tr>\n"
"<tr><td>$area</td><td>area of polygon</td></tr>\n"
"<tr><td>$perimeter</td><td>perimeter of polygon</td></tr>\n"
"<tr><td>$length</td><td>area of line</td></tr>\n"
"<tr><td>$id</td><td>feature id</td></tr>\n"
"<tr><td>$x</td><td>x coordinate of point</td></tr>\n"
"<tr><td>$y</td><td>y coordinate of point</td></tr>\n"
"<tr><td>xat(<tt>n</tt>)</td><td>x coordinate of <tt>n</tt>th point of a line (indices start at 0; negative values apply to the last index)</td></tr>\n"
"<tr><td>yat(<tt>n</tt>)</td><td>y coordinate of <tt>n</tt>th point of a line (indices start at 0; negative values apply to the last index)</td></tr>\n"
"<tr><td><tt>a</tt> ^ <tt>b</tt></td><td><tt>a</tt> raised to the power of <tt>b</tt></td></tr>\n"
"<tr><td><tt>a</tt> * <tt>b</tt></td><td><tt>a</tt> multiplied by <tt>b</tt></td></tr>\n"
"<tr><td><tt>a</tt> / <tt>b</tt></td><td><tt>a</tt> divided by <tt>b</tt></td></tr>\n"
"<tr><td><tt>a</tt> + <tt>b</tt></td><td><tt>a</tt> plus <tt>b</tt></td></tr>\n"
"<tr><td><tt>a</tt> - <tt>b</tt></td><td><tt>a</tt> minus <tt>b</tt></td></tr>\n"
"<tr><td>+<tt>a</tt></td><td>positive sign</td></tr>\n"
"<tr><td>-<tt>a</tt></td><td>negative value of <tt>a</tt></td></tr>\n"
"</table>\n"
"") );
  gContextHelpTexts.insert( "QgsGPSPluginGui", QCoreApplication::translate( "context_help", "<h3>GPS Plugin</h3>\n"
"\n"
"") );
  gContextHelpTexts.insert( "QgsIdentifyResultsDialog", QCoreApplication::translate( "context_help", "<h3>Identify Features</h3>\n"
"<p>\n"
"The results dialog shows all the features identified within the search tolerance. To see the results details, expand the feature information on the left of each feature.\n"
"</p>\n"
"<p>\n"
"The search radius can be set in the options dialog <label>Settings > Options > Map tools</label>, Identify section, as a percentage of map width. In this section you may also set identify mode to <label>Current layer</label>, <label>Top down, stop at first</label>, <label>Top down</label>. <br>\n"
"You may also tick an <label>Open feature form, if a single feature is identified</label> option.<br>\n"
"Identifiable layers can be set in the Project Properties dialog <label>Settings > Project Properties > Identifiable layers</label> where all the layers names are listed, their type (vector, raster, etc.) and ability to be identified.\n"
"</p>\n"
"") );
  gContextHelpTexts.insert( "QgsMeasureDialog", QCoreApplication::translate( "context_help", "<h3>Measure Tools</h3>\n"
"There are three measure tools: length, area and angle. With them, you can measure distances, areas and angles on the map canvas. The tools provide results in the units set in <label>Settings > Options > Map tools</label>.\n"
"<h4>Measuring Lengths</h4>\n"
"To measure lengths, select the tool and click along the path you want to measure. The length of each segment is displayed as is the total length for the path.\n"
"A single right mouse click stops the measuring.\n"
"<h4>Measuring Areas</h4>\n"
"To measure areas, select the tool and click to create the area. The total area is dynamically displayed as you click.\n"
"A single right mouse click stops the measuring, while two right mouse clicks starts measuring a new.\n"
"<h4>Measuring Angles</h4>\n"
"To measure angles, select the tool and click on three points to create an angle between these points. The second point selected is the vertex of the angle. The angle is dynamically displayed once you clicked the second point.\n"
"") );
  gContextHelpTexts.insert( "QgsNewOgrConnection", QCoreApplication::translate( "context_help", "<h3>New OGR Database Connection Dialog</h3>\n"
"") );
  gContextHelpTexts.insert( "QgsNewSpatialiteLayerDialog", QCoreApplication::translate( "context_help", "<h3>Create a New SpatiaLite Layer</h3>\n"
"You can use this dialog to create a new SpatiaLite database and/or an empty SpatiaLite layer for editing. See below for an explanation of the dialog inputs.\n"
"<h4>Database</h4>\n"
"Choose the database from the drop-down list. This list is created from your saved SpatiaLite connections. If you don't have a saved connection or want to create a new database, click on the button (<label>...</label>) to the right of the drop-down.\n"
"<h4>Layer name</h4>\n"
"Enter a name for the layer you want to create. The name should be one word. You can use underscores in the name if you like.\n"
"<h4>Geometry column</h4>\n"
"Enter a name for the geometry column or accept the default.\n"
"<h4>Type</h4>\n"
"Choose the type of layer you want to create. \n"
"<h4>EPSG SRID</h4>\n"
"Enter the EPSG number for the spatial reference id (SRID). By default the SRID for WGS 84 is filled in for you. Click on <label>Find SRID</label> button to change the coordinate reference system of the layer if needed. The SRID must exist within the spatial_ref_sys in your SpatiaLite database. You can search for the SRID using partial matches on both name and SRID.\n"
"<h4>Create an auto-incrementing Primary Key</h4>\n"
"Clicking this check box will add a primary key to the new layer. This key field will be auto-incrementing, meaning you don't have to enter a value for it when adding features to the attribute table of the layer.\n"
"<h4>New attribute</h4>\n"
"Add the desired attributes by clicking on the <label>Add to attributes list</label> button after you have specified a name and type for the attribute. Only real, integer, and string attributes are supported.<br/>\n"
"Width and precision are irrelevant in a SpatiaLite database so you do not have to specify these.\n"
"<h4>Attributes list</h4>\n"
"In this section you can see the list of attributes. To delete one of them, click on it and choose <label>Remove selected attribute</label> button.\n"
"\n"
"<p>\n"
"Click on <label>OK</label> to create the layer and close the dialog.\n"
"<label>Cancel</label> will close the layer without further changes.\n"
"") );
  gContextHelpTexts.insert( "QgsNewVectorLayerDialog", QCoreApplication::translate( "context_help", "<h3>Create New Vector Layer</h3>\n"
"To create a new layer for editing, choose <label>New Vector Layer</label> from the <label>Layer</label> menu.<br/>\n"
"With this dialog you can create a shape file layer.\n"
"<h4>Type</h4>\n"
"Choose the type of layer <label>Point</label>, <label>Line</label> or <label>Polygon</label>.<br/>\n"
"Click on <label>Specify CRS</label> button to change the coordinate reference system of the layer if needed.\n"
"<h4>New attribute</h4>\n"
"Add the desired attributes by clicking on the <label>Add to attributes list</label> button after you have specified a name and type for the attribute. Only real, integer, and string attributes are supported.<br/>\n"
"You can also define the width and precision of the new attribute column.\n"
"<h4>Attributes list</h4>\n"
"In this section you can see the list of attributes. To delete one of them, click on it and choose <label>Remove selected attribute</label> button.\n"
"") );
  gContextHelpTexts.insert( "QgsOpenVectorLayerDialog", QCoreApplication::translate( "context_help", "<h3>Open Vector Layer Dialog</h3>\n"
"\n"
"The open vector layer dialog is used to add vector data to the QGIS map view. Vector data are spatial data described using geometries of points, lines and polygons (enclosed areas).</p>\n"
"QGIS supports vector data in a number of formats, including those supported by the OGR library data provider plugin, such as ESRI shape files, MapInfo MIF (interchange format) and MapInfo TAB (native format).</p>\n"
"QGIS supports PostGIS layers in a PostgreSQL database and SpatiaLite layers. Support for additional data types (eg. delimited text) is provided by additional data provider plugins. A list of OGR supported vector formats can be found in Appendix A.1 of the User Guide.</p>\n"
"Detailed description how to proceed with vector data is described in chapter 5 'Working with Vector Data'.\n"
"") );
  gContextHelpTexts.insert( "QgsOptions", QCoreApplication::translate( "context_help", "<h3>Options</h3>\n"
"This dialog allows you to select some basic options for QGIS. There are eight tabs:\n"
"<p>\n"
"\n"
"<a href=\"#general\">General</a><br/>\n"
"<a href=\"#ren\">Rendering & SVG</a><br/>\n"
"<a href=\"#map\">Map tools</a><br/>\n"
"<a href=\"#over\">Overlay</a><br/>\n"
"<a href=\"#digit\">Digitizing</a><br/>\n"
"<a href=\"#crs\">CRS</a><br/>\n"
"<a href=\"#loc\">Locale</a><br/>\n"
"<a href=\"#net\">Network & Proxy</a><br/>\n"
"\n"
"<a name=\"general\">\n"
"<h4>General</h4>\n"
"</a>\n"
"On this tab you can set general options for project behavior on opening/saving, default map appearance, your favorite icon theme and many others.\n"
"<a name=\"ren\">\n"
"<h4>Rendering & SVG</h4>\n"
"</a>\n"
"Within this tab you can set general rendering options for map layers.<br>\n"
"By default, QGIS renders all visible layers whenever the map canvas must be refreshed. The events that trigger a refresh of the map canvas include:\n"
"<ul>\n"
"<li>Adding a layer\n"
"<li>Panning or zooming\n"
"<li>Resizing the QGIS window\n"
"<li>Changing the visibility of a layer or layers\n"
"</ul>\n"
"QGIS allows you to control the rendering process in a number of ways:<p>\n"
"You can set an option to always load new layers without drawing them.<br>\n"
"This means the layer will be added to the map, but its visibility check box in the legend will be unchecked by default. To set this option, uncheck the <label>By default new layers added to the map should be displayed</label> check box.<p>\n"
"You can set an option to update the map display as features are drawn. <br>\n"
"By default, QGIS does not display any features for a layer until the entire layer has been rendered. To update the display as features are read from the data store, set the <label>Number of features to draw before updating the display</label> to an appropriate value. Setting a value of 0 disables update during drawing (this is the default). Setting a value too low will result in poor performance as the map canvas is continually updated during the reading of the features. A suggested value to start with is 500.\n"
"<a name=\"map\">\n"
"<h4>Map tools</h4>\n"
"</a>\n"
"<h5>Identify</h5>\n"
"The <label>Mode</label> setting determines which layers will be shown by the <label>Identify</label> tool. By switching to <label>Top down</label> instead of <label>Current layer</label> attributes for all identifiable layers will be shown with the <label>Identify</label> tool.\n"
"<h5>Measure tool</h5>\n"
"Here you can set an ellipsoid for distance calculations, measurement units and preferred color of rubber band.\n"
"<h5>Panning and zooming</h5>\n"
"Allows defining mouse wheel action and zoom factor.\n"
"<a name=\"over\">\n"
"<h4>Overlay</h4>\n"
"</a>\n"
"Define placement algorithm for labels. Choose between <label>central point (fastest)</label>, <label>chain (fast)</label>, <label>popmusic tabu chain (slow)</label>, <label>popmusic tabu (slow)</label> and <label>popmusic chain (very slow)</label>.\n"
"\n"
"<a name=\"digit\">\n"
"<h4>Digitizing</h4>\n"
"</a>\n"
"<h5>Rubber band</h5>\n"
"In the <label>Digitizing</label> tab you can define settings for digitizing line width and color.\n"
"<h5>Snapping</h5>\n"
"Here you can define a general, project wide snapping tolerance. <p>\n"
"You can select between <label>To vertex</label>, <label>To segment</label> or <label>To vertex and segment</label> as default snap mode. You can also define a default snapping tolerance and a search radius for vertex edits. The tolerance can be set either in map units or in pixels. The advantage of choosing pixels, is that the snapping tolerance doesn't have to be changed after zoom operations.<p>\n"
"A layer based snapping tolerance can be defined by choosing <label>Settings</label> (or <label>File</label>) > <label>Project Properties...</label>. In the <label>General</label> tab, section <label>Digitize</label> you can click on <label>Snapping options...</label> to enable and adjust snapping mode and tolerance on a layer basis.\n"
"<h5>Vertex markers</h5>\n"
"You can define marker style as <label>Semi transparent circle</label>, <label>Cross</label> or <label>None</label> and size; you can also set showing markers only for selected features.\n"
"<h5>Enter attribute values</h5>\n"
"By default, after digitizing a new feature, QGIS asks you to enter attribute values for that feature. Ticking <label>Suppress attributes pop-up windows after each created feature</label> allows you to enter the attributes later.\n"
"<a name=\"crs\">\n"
"<h4>CRS</h4>\n"
"</a>\n"
"QGIS no longer sets the map CRS to the coordinate reference system of the first layer loaded. When you start a QGIS session with layers that do not have a CRS, you need to check and define the CRS definition for these layers. This can be done globally on this tab.<p>\n"
"The global default CRS <i>proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs</i> comes predefined in QGIS but can of course be changed, and the new definition will be saved for subsequent QGIS sessions.\n"
"<a name=\"lov\">\n"
"<h4>Locale</h4>\n"
"</a>\n"
"Informs you about active system locale. To change it tick <label>Override system locale</label> and restart QGIS.\n"
"<a name=\"net\">\n"
"<h4>Network & Proxy</h4>\n"
"</a>\n"
"Tick <label>Use proxy for web access</label> to define host, port, user, and password.\n"
"") );
  gContextHelpTexts.insert( "QgsOracleNewConnection", QCoreApplication::translate( "context_help", "<h3>Create a New Oracle Connection</h3>\n"
"This dialog allows you to define the settings for a connection to a Oracle database.\n"
"<p>\n"
"<ul>\n"
"<li> <label>Name</label> A name to identify the connection settings.\n"
"\n"
"<li> <label>Database</label> SID or SERVICE_NAME of the Oracle instance.\n"
"\n"
"<li> <label>Host</label> Name or IP address of the computer hosting the database server\n"
"\n"
"<li> <label>Port</label> IP port used by the database server\n"
"\n"
"<li> <label>Username</label> Database user name.\n"
"<li> <label>Password</label> Database password.\n"
"\n"
"<li> <label>Save Username</label> Indicates whether to save the database user name in the connection configuration.\n"
"\n"
"<li> <label>Save Password</label> Indicates whether to save the database password in the connection settings.  <font color=\"red\">Passwords are saved in <strong>clear text</strong> in the system configuration and in the project files!</font>\n"
"\n"
"<li> <label>Only look in meta data table</label> Restricts the displayed tables to those that are in the all_sdo_geom_metadata view. This can speed up the initial display of spatial tables.\n"
"\n"
"<li> <label>Only look for user's tables</label> When searching for spatial tables restrict the search to tables that are owner by the user.\n"
"\n"
"<li> <label>Also list tables with no geometry</label>  Indicates that tables without geometry should also be listed by default.\n"
"\n"
"<li> <label>Use estimated table statistics for the layer metadata</label> When\n"
"the layer is setup various metadata is required for the Oracle table. This\n"
"includes information such as the table row count, geometry type and spatial\n"
"extents of the data in the geometry column. If the table contains a large\n"
"number of rows determining this metadata is time consuming. By activating this\n"
"option the following fast table metadata operations are done: Row count is\n"
"determined from all_tables.num_rows. Table extents are always determined with\n"
"the SDO_TUNE.EXTENTS_OF function even if a layer filter is applied. The table\n"
"geometry is determined from the first 100 non-null geometry rows in the table.\n"
"\n"
"<li> <label>Only existing geometry types</label> Only list the existing\n"
"geometry types and don't offer to add others.\n"
"\n"
"</ul>\n"
"") );
  gContextHelpTexts.insert( "QgsPgNewConnection", QCoreApplication::translate( "context_help", "<h3>Create a New PostGIS Connection</h3>\n"
"This dialog allows you to define the settings for a connection to a PostgreSQL/PostGIS database.\n"
"<p>\n"
"<ul>\n"
"<li> <label>Name</label> A name to identify the connection settings.\n"
"\n"
"<li> <label>Service</label> A name of a service listed in the <a href=\"http://www.postgresql.org/docs/current/static/libpq-pgservice.html\">Connection Service File</a>.\n"
"\n"
"<li> <label>Host</label> Name or IP address of the computer hosting the database server (leave blank for local connections or services).\n"
"\n"
"<li> <label>Port</label> IP port used by the database server (leave blank for local connections or to use default port 5432).\n"
"\n"
"<li> <label>Database</label> Name of the database (leave blank for default database).\n"
"\n"
"<li> <label>SSL mode</label> <a href=\"http://www.postgresql.org/docs/9.0/static/libpq-ssl.html#LIBPQ-SSL-SSLMODE-STATEMENTS\">SSL mode</a> of the connection\n"
"\n"
"<li> <label>Username</label> Database user name.\n"
"<li> <label>Password</label> Database password.\n"
"\n"
"<li> <label>Save Username</label> Indicates whether to save the database user name in the connection configuration.\n"
"\n"
"<li> <label>Save Password</label> Indicates whether to save the database password in the connection settings.  <font color=\"red\">Passwords are saved in <strong>clear text</strong> in the system configuration and in the project files!</font>\n"
"\n"
"<li> <label>Only look in the geometry_columns table</label> Indicates that only \"listed\" geometry columns should be used - opposed to scanning all tables for geometry columns.\n"
"\n"
"<li> <label>Only look in the 'public' schema</label> Indicates that only tables in the 'public' schema should be considers - opposed to all accessible tables.\n"
"\n"
"<li> <label>Also list tables with no geometry</label>  Indicates that tables without geometry should also be listed by default.\n"
"\n"
"<li> <label>Use estimated table metadata</label> When initializing layers, various queries may be needed to establish the characteristics of the geometries stored in the database table.  When this option is checked, these queries examine only a sample of the rows and use the table statistics, rather than the entire table.  This can drastically speed up operations on large datasets, but may result in incorrect characterization of layers (eg. the <font color=\"red\">feature count</font> of filtered layers will not be accurately determined).\n"
"\n"
"</ul>\n"
"") );
  gContextHelpTexts.insert( "QgsPgSourceSelect", QCoreApplication::translate( "context_help", "<h3>Add PostGIS Tables</h3>\n"
"This dialog allows you to add PostGIS layers (tables with a geometry column) to the QGIS map canvas.\n"
"<p>\n"
"<a href=\"#connect\">Connections</a><br/>\n"
"<a href=\"#add\">Adding Layers</a><br/>\n"
"<a href=\"#filter\">Filtering a Layer</a><br/>\n"
"<a href=\"#search\">Search options</a><br/>\n"
"\n"
"<a href=\"#connect\">\n"
"<h4>Connections</h4>\n"
"</a>\n"
"<ul>\n"
"<li>Choose the connection to use from the drop-down box and click <label>Connect</label>.\n"
"<li>If there are no connections, use the <label>New</label> button to create a connection.\n"
"<li>To modify a connection, click the <label>Edit</label> button.\n"
"</ul>\n"
"<a name=\"add\">\n"
"<h4>Adding Layers</h4>\n"
"</a>\n"
"To add a layer:\n"
"<ol>\n"
"<li>Choose the desired connection from the drop-down box.\n"
"<li>Click <label>Connect</label>.\n"
"<li>Find the layer you want to add in the list and click on it to select it.\n"
"<li>You can select additional layers by holding down the Ctrl key and clicking.\n"
"<li>Click <label>Add</label> to add the layer(s) to the map.\n"
"</ol>\n"
"<a name=\"filter\">\n"
"<h4>Filtering a Layer</h4>\n"
"</a>\n"
"To filter a layer before adding it to the map, double click on its name or use <label>Build query</label> button. This will open the Query Builder, allowing you to build up a SQL statement to use in filtering the records.\n"
"\n"
"<a name=\"search\">\n"
"<h4>Search options</h4>\n"
"</a>\n"
"Ticking <label>Search options</label> enables additional options for searching in different column types and using 2 search modes: <label>Wildcard</label> or <label>RegExp</label>.\n"
"") );
  gContextHelpTexts.insert( "QgsPluginManager", QCoreApplication::translate( "context_help", "<h3>Manage and install Plugins</h3>\n"
"\n"
"<p>\n"
"Manage and install plugins.\n"
"</p>\n"
"\n"
"\n"
"<a href=\"#installedtab\">Installed tab</a><br/>\n"
"<a href=\"#getmoretab\">Get more tab</a><br/>\n"
"<a href=\"#newtab\">New tab</a><br/>\n"
"<a href=\"#upgradabletab\">Upgradable tab</a><br/>\n"
"<a href=\"#invalidtab\">Invalid tab</a><br/>\n"
"<a href=\"#settingstab\">Settings tab</a><br/>\n"
"\n"
"<a name=\"installedtab\">\n"
"<h4>Installed tab</h4>\n"
"</a>\n"
"<p>\n"
"This tab shows you the list of currently installed plugins on your system. Both\n"
"python and C++ plugins are listed.\n"
"</p>\n"
"<p>\n"
"You can enable or disable them by clicking the checkbox in front of the name.<br/>\n"
"Selecting an item show the plugin information in the detail pane.\n"
"</p>\n"
"<p>\n"
"There is a context menu (right click) available, to change the sorting on:\n"
"name, number of downloads, votes and status.\n"
"</p>\n"
"\n"
"\n"
"<a name=\"getmoretab\">\n"
"<h4>Get more tab</h4>\n"
"</a>\n"
"<p>\n"
"This tab shows you the list of available plugins from the remote repositories.\n"
"It is downloaded once if you start up. But you can change that in the Settings\n"
"</p>\n"
"<p>\n"
"Selecting an item shows you the plugin information in the detail pane.\n"
"</p>\n"
"\n"
"<a name=\"newtab\">\n"
"<h4>New tab</h4>\n"
"</a>\n"
"<p>\n"
"This tab will be shown when a new plugin is made available! It will be shown\n"
"just once.\n"
"</p>\n"
"\n"
"<a name=\"upgradabletab\">\n"
"<h4>Upgradable tab</h4>\n"
"</a>\n"
"<p>\n"
"This tab is only available if one of your installed plugins can be upgraded\n"
"from the remote repositories.\n"
"</p>\n"
"\n"
"<a name=\"Invalid tab\">\n"
"<h4>Invalid tab</h4>\n"
"</a>\n"
"<p>\n"
"This tab is only available if there is a invalid plugin. Mostly caused by an error \n"
"during loading the plugin.\n"
"</p>\n"
"\n"
"<a name=\"settingstab\">\n"
"<h4>Settings tab</h4>\n"
"</a>\n"
"<p>\n"
"In this tab you can change the Settings for the plugin manager.\n"
"</p>\n"
"<p>\n"
"It is possible to check how often QGIS will download the list and update information \n"
"from the available repositories.\n"
"</p>\n"
"<p>\n"
"By checking the 'Show experimental plugins' checkbox, all plugins will be shown\n"
"in the lists, even the ones that are considered 'experimental' by the author.\n"
"\n"
"<table border=1 width='95%' align='center'><tr><td>\n"
"Note: Experimental plugins are generally unsuitable for production use. These plugins are in the early stages of development, and should be considered \"incomplete\" or \"proof of concept\" tools. The QGIS development team does not recommend installing these plugins unless you intend to use them for testing purposes.\n"
"</td></tr></table>\n"
"\n"
"</p>\n"
"") );
  gContextHelpTexts.insert( "QgsProjectProperties", QCoreApplication::translate( "context_help", "<h3>Project Properties</h3>\n"
"This dialog allows you to set a number of properties related to the <label>Project</label>. A Project is a saved set of layers and their settings, as well as other information required to restore the project from disk.<p>\n"
"\n"
"<a href=\"#general\">General Tab</a><br/>\n"
"<a href=\"#crs\">Coordinate Reference System (CRS) Tab</a><br/>\n"
"<a href=\"#layers\">Identifiable layers Tab</a><br/>\n"
"\n"
"<a name=\"general\">\n"
"<h4>General Tab</h4>\n"
"</a>\n"
"<table border=1>\n"
"<tr><td><label>Project title</label></td><td>A descriptive title for the project. This title will be displayed in the title bar of the QGIS window</td></tr>\n"
"<tr><td><label>Selection color</label></td><td>Selected features are displayed on the map using this color.</td></tr>\n"
"<tr><td><label>Background color</label></td><td>The color of the map canvas.</td></tr>\n"
"<tr><td><label>Save paths</label></td><td>Ability to save the paths as absolute or relative.</td></tr>\n"
"<tr><td><label>Layer units</label></td><td>Choose the units appropriate for the layers in the project.</td></tr>\n"
"<tr><td><label>Precision</label></td><td>Use the manual setting to set the precision of the display in the status bar. As the mouse is moved over the map canvas, the coordinates are updated and displayed in the status bar using the precision you choose. Automatic (the default) results in a default precision based on the map units.</td></tr>\n"
"<tr><td><label>Digitizing</label></td><td>The option <label> Enable topological editing</label> enables editing and maintaining common boundaries in polygon mosaics. Editing a vertex in shared boundary will result in updating both boundaries.<p>\n"
"To avoid digitizing all vertices of the common boundary select <label>Avoid intersections of new polygons</label>. In the dialog window tick appropriate layers to which the new polygon will be adjusted. Draw the new polygon in the way it overlap the existing one, it will be cut automatically. <p>\n"
"<label>Snapping options</label> allows you to set the snapping tolerance in pixels or map units and the snapping mode: <label>to vertex</label>, <label>to segment</label>, <label>to vertex and segment</label> for every layer separately.</td></tr>\n"
"\n"
"</table>\n"
"<a name=\"crs\">\n"
"<h4>Coordinate Reference System (CRS) Tab</h4>\n"
"</a>\n"
"The Coordinate Reference System (CRS) tab allows you to set the projection for the current project and enable on the fly transformation. You do not need to have a saved project to setup and enable 'on the fly' CRS transformation.\n"
"<p>To use 'on the fly' CRS transformation, click the <label>Enable 'on the fly' CRS transformation</label> check box. Then choose the projection for the project (map canvas) from the list of Geographic or Projected coordinate systems.\n"
"<p>You can find a coordinate system by entering its name, EPSG code or the QGIS ID and clicking the <label>Find</label> button.\n"
"<p>The list of recently used coordinate systems will be available on the bottom for quick access.\n"
"\n"
"<a name=\"layers\">\n"
"<h4>Identifiable layers Tab</h4>\n"
"</a>\n"
"On this tab you can choose on which layers the identify tool will work.\n"
"") );
  gContextHelpTexts.insert( "QgsQueryBuilder", QCoreApplication::translate( "context_help", "<h3>Query Builder</h3>\n"
"The query builder allows you to create provider specific filter expressions.\n"
"\n"
"These are usually evaluated much faster than the generic QGIS expressions, but\n"
"are sometimes limited (e.g. for shape files) or behave diffently.\n"
"\n"
"<h4>Providers</h4>\n"
"\n"
"<table border=1>\n"
"<tr>\n"
"  <th>Provider</th>\n"
"  <th>Documentation</th>\n"
"</tr>\n"
"<tr>\n"
"  <td>OGR</td>\n"
"  <td>where clauses depending on the\n"
"    <a href=\"http://www.gdal.org/ogr/ogr_formats.html\">format</a> (eg.\n"
"    <a href=\"http://www.gdal.org/ogr/ogr_sql.html\">OGR SQL</a>\n"
"    for shape files)\n"
"  </td>\n"
"</tr>\n"
"<tr>\n"
"  <td>PostgreSQL/PostGIS</td>\n"
"  <td>where clauses using\n"
"    <a href=\"http://www.postgresql.org/docs/\">PostgreSQL</a> and\n"
"    <a href=\"http://postgis.net/documentation\">PostGIS</a> \n"
"  </td>\n"
"</tr>\n"
"<tr>\n"
"  <td>SpatiaLite</td>\n"
"  <td>where clauses using\n"
"    <a href=\"http://www.sqlite.org/lang_select.html\">SQLite</a> and\n"
"    <a href=\"http://www.gaia-gis.it/gaia-sins/spatialite-sql-4.1.0.html\">SpatiaLite</a>\n"
"  </td>\n"
"</tr>\n"
"<tr>\n"
"  <td>Oracle Spatial/Locator</td>\n"
"  <td>where clauses using\n"
"    <a href=\"http://docs.oracle.com/cd/B28359_01/appdev.111/b28400/sdo_index_query.htm#autoId9\">Oracle Spatial/Locator</a>\n"
"  </td>\n"
"</tr>\n"
"<tr>\n"
"  <td colspan=2>...</td>\n"
"</tr>\n"
"</table>\n"
"") );
  gContextHelpTexts.insert( "QgsSpatiaLiteSourceSelect", QCoreApplication::translate( "context_help", "<h3>Add SpatiaLite Tables</h3>\n"
"To define a new connection, click on <label>New</label> button and use the file browser to point to your SpatiaLite database, which is a file with a <i>.sqlite</i> extension.<p>\n"
"The connections you define will be remembered for further use and available to choose from the drop down menu.\n"
"") );
  gContextHelpTexts.insert( "QgsStyleV2ManagerDialog", QCoreApplication::translate( "context_help", "<h3>Style Manager</h3>\n"
"") );
  gContextHelpTexts.insert( "QgsVectorLayerProperties", QCoreApplication::translate( "context_help", "<h3>Vector Layer Properties</h3>\n"
"This dialog allows you to work with vector layer settings and properties. There are seven tabs:\n"
"<p>\n"
"<a href=\"#symbology\">Symbology</a><br/>\n"
"<a href=\"#labels\">Labels</a><br/>\n"
"<a href=\"#attributes\">Attributes</a><br/>\n"
"<a href=\"#general\">General</a><br/>\n"
"<a href=\"#metadata\">Metadata</a><br/>\n"
"<a href=\"#actions\">Actions</a><br/>\n"
"<a href=\"#diagram\">Diagram Overlay</a><br/>\n"
"\n"
"<a name=\"symbology\">\n"
"<h4>Symbology</h4>\n"
"</a>\n"
"To change the symbology for a layer, simply double click on its legend entry and the vector <label>Layer Properties</label> dialog will be shown.<p>\n"
"Within this dialog you can style your vector layer. Depending on the selected rendering option you have the possibility to also classify your map features.\n"
"\n"
"<h5>Legend type</h5>\n"
"QGIS supports a number of symbology renderers to control how vector features are displayed. Currently the following renderers are available:\n"
"<ul>\n"
"<li><label>Single symbol</label> - a single style is applied to every object in the layer.\n"
"<li><label>Graduated symbol</label> - objects within the layer are displayed with different symbols classified by the values of a particular field.\n"
"<li><label>Continuous color</label> - objects within the layer are displayed with a spread of colors classified by the numerical values within a specified field.\n"
"<li><label>Unique value</label> - objects are classified by the unique values within a specified field with each value having a different symbol.\n"
"</ul>\n"
"\n"
"<h5>New symbology</h5>\n"
"Click <label>New symbology</label> button to use new symbology implementation for the layer.\n"
"\n"
"<h5>Vector transparency</h5>\n"
"QGIS allows setting a transparency for every vector layer. This can be done with the slider <label>Transparency</label>. This is very useful for overlaying several vector layers.\n"
"\n"
"<h5>Saving styles</h5>\n"
"Once you have styled your layer you also could save your layer-style to a separate file (with *.qml ending). To do this, use the button <label>Save Style...</label>. No need to say that <label>Load Style...</label> loads your saved layer-style-file.<br/>\n"
"If you wish to always use a particular style whenever the layer is loaded, use the <label>Save As Default</label> button to make your style the default. Also, if you make changes to the style that you are not happy with, use the <label>Restore Default Style</label> button to revert to your default style.\n"
"\n"
"<a name=\"labels\">\n"
"<h4>Labels</h4>\n"
"</a>\n"
"The Labels tab allows you to enable labeling features and control a number of options related to fonts, placement, style, alignment and buffering.<br/>\n"
"Check the <label>Display labels</label> check box to enable labeling.\n"
"\n"
"<h5>Basic Label options</h5>\n"
"Choose the field to label with.<p>\n"
"Select the use the <label>Font</label> and  <label>Color</label> buttons to set the font and color. You can also change the angle and the placement of the text-label.<p>\n"
"If have labels extending over several lines, check <label>Multiline labels?</label>. QGIS will check for a true line return in your label field and insert the line breaks accordingly. A true line return is a single character &#92;n, (not two separate characters, like a backslash &#92; followed by the character n).\n"
"\n"
"<h5>Placement</h5>\n"
"Change the label placement by selecting one of the radio buttons in the <label>Placement</label> group.\n"
"\n"
"<h5>Font size units</h5>\n"
"The <label>Font size units</label> allows you to select between <label>Points</label> or <label>Map units</label>.\n"
"\n"
"<h5>Buffer labels</h5>\n"
"To buffer the labels means putting a backdrop around them to make them stand out better. To buffer the lakes labels:\n"
"<ol>\n"
"<li>Click the <label>Buffer Labels</label> check box to enable buffering.\n"
"<li>Choose a size for the buffer using the spin box.\n"
"<li>Choose a color by clicking on <label>Color</label> and choosing your favorite from the color selector. You can also set some transparency for the buffer if you prefer.\n"
"<li>Click <label>Apply</label> to see if you like the changes.\n"
"</ol>\n"
"Notice you can also specify the buffer size in map \n"
"units if that works out better for you.\n"
"\n"
"<h5>Data defined placement, properties, buffer, position</h5>\n"
"The remaining entries inside the <label>Label</label> tab allow you control the appearance of the labels using attributes stored in the layer. The entries beginning with <label>Data defined</label> allow you to set all the parameters for the labels using fields in the layer.\n"
"\n"
"\n"
"<a name=\"attributes\">\n"
"<h4>Attributes</h4>\n"
"</a>\n"
"Within the <label>Attributes</label> tab the attributes of the selected data set can be manipulated. The buttons <label>New Column</label> and <label>Delete Column</label> can be used, when the data set is in editing mode. The OGR library supports to add new columns, but not to remove them, if you have a GDAL version >= 1.6 installed. Otherwise only columns from PostGIS layers can be edited.<p>\n"
"The <label>Toggle editing mode</label> button toggles this mode.<p>\n"
"Within the Attributes tab you also find an edit widget column. This column can be used to define values or a range of values that are allowed to be added to the specific attribute table column. If you click on the <label>edit widget</label> button, a dialog opens, where you can define different widgets. These widgets are:<br/>\n"
"<ul>\n"
"<li><label>Line edit</label> an edit field which allows entering simple text (or restrict to numbers for numeric attributes).\n"
"<li><label>Classification</label> Displays a combo box with the values used for classification, if you have chosen <label>unique value</label> as legend type in the symbology tab of the properties dialog.\n"
"<li><label>Range</label> Allows setting of numeric values from a specific range. The edit widget can be either a slider or a spin box.\n"
"<li><label>Unique value</label> The user can select one of the values already used in the attribute table. If <label>editable</label> is activated, a line edit is shown with auto completion support, but additional values are possible, otherwise a combo box is used.\n"
"<li><label>File name</label> Simplifies the selection by adding a file chooser dialog.\n"
"<li><label>Value map</label> a combo box with predefined items. The value is stored in the attribute, the description is shown in the combo box. You can define values manually or load them from a layer or a csv file.\n"
"<li><label>Enumeration</label> Opens a combo box with values that can be used within the columns type. This is currently only supported by the Postgres provider.\n"
"<li><label>Immutable</label> The immutable attribute column is read-only. The user is not able to modify the content.\n"
"<li><label>Hidden</label> A hidden attribute column is invisible to the user.\n"
"<li><label>Checkbox</label> A check box is shown.  The value representing the state of the check box in the attribute has to be entered.\n"
"<li><label>Text edit</label> A text edit field is shown, that allows entering multiline text.\n"
"</ul>\n"
"\n"
"<a name=\"general\">\n"
"<h4>General</h4>\n"
"</a>\n"
"The General tab allows you to change the display name, set scale dependent rendering options, create a spatial index of the vector file (only for OGR supported formats and PostGIS) and view or change the projection of the specific vector layer.<p>\n"
"You can also set a provider specific filter expressions using the <label>Query Builder</label>.\n"
"\n"
"<h5>Subset</h5>\n"
"The <label>Query Builder</label> button allows you to limit the features of a layer to a subset matching an provider specific filter expression.\n"
"\n"
"<a name=\"metadata\">\n"
"<h4>Metadata</h4>\n"
"</a>\n"
"The <label>Metadata</label> tab contains information about the layer, including specifics about the type and location, number of features, feature type, and the editing capabilities. The Layer Spatial Reference System section, providing projection information, is displayed on this tab. This is a quick way to get information about the layer.\n"
"\n"
"<a name=\"actions\">\n"
"<h4>Actions</h4>\n"
"</a>\n"
"QGIS provides the ability to perform an action based on the attributes of a feature. This can be used to perform any number of actions, for example, running a program with arguments built from the attributes of a feature or passing parameters to a web reporting tool. <br/>\n"
"Actions are useful when you frequently want to run an external application or view a web page based on one or more values in your vector layer.<p>\n"
"\n"
"The new implementation of actions uses the QGIS expression engine to do evaluations. Each expression is evaluated and replaced by its result at runtime.<p>\n"
"An expression must be enclosed between <code>[%</code> and <code>%]</code> - the GUI has some buttons which do that for you automatically. \n"
"The <label>Insert expression...</label> button launches the <label>Expression builder</label> which allows writing an expression easily, the <label>Insert field</label> button inserts into the action a placeholder like <code>[% \"fieldname\" %]</code> where fieldname is the name of the field selected in the field dropdown list.<p>\n"
"\n"
"See the User Guide for further information.\n"
"\n"
"<h5>Using Actions</h5>\n"
"Actions can be invoked from the <label>Identify Results</label> dialog or using the <label>Run feature action</label> tool on the toolbar.<p>\n"
"\n"
"Each action adds a little set of custom expressions to the default set available in the <label>Expression builder</label>.<br/>\n"
"While running actions from the <label>Identify Results</label> dialog the custom expression <code>$currfield</code> will be replaced with the value of the selected field in the dialog, using the <label>Run feature action</label> tool the following custom expressions are available (instead of $currfield):\n"
"<ul>\n"
"<li><code>$clickx</code> returns the x coordinate of the click position on the canvas</li>\n"
"<li><code>$clicky</code> returns the y coordinate of the click position on the canvas</li>\n"
"<li><code>$layerid</code> returns the ID of the selected layer in the legend</li>\n"
"</ul>\n"
"\n"
"Note: the <label>Run feature action</label> tool executes the actions on all the matching features, where as <label>Identify Results</label> allows you to select which specific feature to run action on.<p>\n"
"\n"
"\n"
"<h5>Action Examples</h5>\n"
"You can add some example actions by clicking on the <label>Add default actions</label> button.\n"
"\n"
"\n"
"<a name=\"diagram\">\n"
"<h4>Diagram Overlay</h4>\n"
"</a>\n"
"The Diagram tab allows you to add a graphic overlay to a vector layer. To activate this feature, open the <label>Plugin Manager</label> and select the <label>Diagram Overlay</label> plugin. After this, there is a new tab in the vector <label>Layer Properties</label> dialog where the settings for diagrams may be entered.<p>\n"
"The current implementation of diagrams provides support for pie- and bar charts and for linear scaling\n"
"of the diagram size according to a classification attribute.\n"
"") );
  gContextHelpTexts.insert( "QgsVectorLayerSaveAsDialog", QCoreApplication::translate( "context_help", "<h3>Save vector layer as...</h3>\n"
"\n"
"<p>This dialog allows you to save vector data in various formats using GDAL/OGR.\n"
"\n"
"<ul>\n"
"<li>From the <label>Format</label> list you can select the destination format (as advertised by OGR).\n"
"<li>At <label>Save as</label> you can enter a destination files name or select one using the <label>Browse</label> button.\n"
"<li>In the <label>Encoding</label> list you can define in which encoding the data should be saved.\n"
"<li>Using the <label>CRS</label> you can select a CRS into which the data about to be saved should be reprojected.\n"
"<li>OGR also has various options for the different formats it supports.  Use the <label>datasource</label> creation field to set the datasource options and the <label>layer</label> creation options. Enter one options per line (e.g. <code>SPATIALITE=yes</code> in the <label>datasource</label> to create a spatialite database using the SQLite driver).\n"
"</ul>\n"
"\n"
"See <a href=\"http://gdal.org/ogr/ogr_formats.html\">OGR Vector formats</a> for a list of supported formats and the available options.\n"
"") );
  gContextHelpTexts.insert( "QgsWMSSourceSelect", QCoreApplication::translate( "context_help", "<h3>Create a new WMS connection</h3>\n"
"WMS layers can be added quite simply, as long as you know the URL to access the WMS server, you have a serviceable connection to that server, and the server understands HTTP as the data transport mechanism.<p>\n"
"QGIS currently can act as a WMS client that understands WMS 1.1, 1.1.1 and 1.3 servers.\n"
"<p>\n"
"<a href=\"#serv\">Servers</a><br/>\n"
"<a href=\"#load\">Loading WMS Layers</a><br/>\n"
"<a href=\"#lay\">Layer Order</a><br/>\n"
"<a href=\"#search\">Server Search</a><br/>\n"
"\n"
"<a name=\"serv\">\n"
"<h4>Servers</h4>\n"
"</a>\n"
"To define a new WMS server in the <label>Server Connections</label> section, select <label>New</label>. Then enter the parameters to connect to your desired WMS server.<p>\n"
"You can add some servers to play with by clicking the <label>Add default servers</label> button. This will add at least three WMS servers for you to use.<p>\n"
"Once the new WMS Server connection has been created, it will be preserved for future QGIS sessions.\n"
"\n"
"<a name=\"load\">\n"
"<h5>Loading WMS Layers</h5>\n"
"</a>\n"
"Select the <label>Connect</label> button to retrieve the capabilities of the selected server. This includes the Image encoding, Layers, Layer Styles and Projections.<br/>\n"
"While downloading data from the WMS server, the download progress is visualized in the left bottom of the WMS Plugin dialog.<p>\n"
"The <label>Image encoding</label> section now lists the formats that are supported by both the client and server.<br/>\n"
"Choose one depending on your image accuracy requirements.<p>\n"
"You can select several layers at once, they will be combined at the WMS Server and transmitted to QGIS in one go.<p>\n"
"Each WMS Layer can be presented in multiple CRSs, depending on the capability of the WMS server. To choose a CRS, select <label>Change...</label>\n"
"\n"
"<a name=\"lay\">\n"
"<h4>Layer Order</h4>\n"
"</a>\n"
"On this tab you can change the order of loaded WMS layers.\n"
"\n"
"<a name=\"search\">\n"
"<h4>Server Search</h4>\n"
"</a>\n"
"On this tab you can search for WMS servers. It is possible to enter a search-string in the text field an hit the <label>Search</label> button.<p>\n"
"To visualize the results, select an table entry, press the <label>Add selected row to WMS list</label> button and change back to the <label>Servers</label> tab.<p>\n"
"You only need to request the list of layers by clicking the <label>Connect</label> button.\n"
"") );
  gContextHelpTexts.insert( "RgSettingsDlg", QCoreApplication::translate( "context_help", "<h3>Road graph plugin settings</h3>\n"
"<p>Road graph is a C++ plugin for QGIS, that calculates the shortest path between two points on any polyline layer and plots this path over the road network.</p>\n"
"<p>\n"
"<a href=\"#creating\">Plugin units</a><br/>\n"
"<a href=\"#topologyTolerance\">Topology tolerance</a><br/>\n"
"<a href=\"#layerSettings\">Transport layer settings</a><br/>\n"
"</p>\n"
"\n"
"<a name=\"creating\">\n"
"<h4>Plugin units</h4>\n"
"</a>\n"
"<p>You can adjust the units of the calculated distance and time of the path. Select the units in <label>Distance unit</label> and <label>Time unit</label>.</p>\n"
"\n"
"<a name=\"topologyTolerance\">\n"
"<h4>Topology tolerance</h4>\n"
"</a>\n"
"<p><label>Topology tolerance</label> - the distance in units of the project. If the distance of two points falls short the topological tolerance, they are considered to be the vertex. A topological tolerance value greater than zero slows down the plugin, but you can use not ideal data sources.</p>\n"
"\n"
"<h5>Note</h5>\n"
"<p>The Road graph plugin considers two roads as linked, if they have a common node. A Node is uniquely identified by it's coordinates. However, in the process of digitizing errors may occur. The errors sometimes occur when the network is transformed from one coordinate reference system to another. This might cause the Road graph plugin to consider connected roads as disconnected. In that case the topological tolerance should be set to a value greater than zero.</p>\n"
"\n"
"<a name=\"LayerSettings\">\n"
"<h4>Transport layer settings</h4>\n"
"</a>\n"
"<p><label>Layer</label> - the data layer of roads.</p>\n"
"<p><label>Field direction</label> - the value of this field indicates how to interpret the Road graph plugin layers feature. <label>Value for forward direction</label> - the direction of movement corresponds to the order of points in a feature. <label>Value for reverse direction</label> - the direction of movement corresponds to the reverse order of points in the feature. <label>Value for two-way direction</label> - both possible.</p>\n"
"<p>Speed field</p> - field containing the recorded speed on the road.</p>\n"
"\n"
"<h5>Default settings</h5>\n"
"<p>If the attribute table of your linear layer does not contain the required fields or fields contain unexpected values, the plugin will use default values. You can set them in the default tab <label>Default settings</label>.</p>\n"
"") );
  gContextHelpTexts.insert( "RgShortestPathWidget", QCoreApplication::translate( "context_help", "<h3>Finding shortest path</h3>\n"
"<p>Road graph is a C++ plugin for QGIS, that calculates the shortest path between two points on any line layer and plots this path over the road network. Before using the plugin to configure it. This can be done in the menu <label>Vector &rarr; Road graph &rarr; Settings</label></p>\n"
"<p>\n"
"<a href=\"#howto\">How to</a><br/>\n"
"</p>\n"
"\n"
"<a name=\"howto\">\n"
"<h4>How to</h4>\n"
"</a>\n"
"<p>You can select start and end points with the select buttons next to the fields.\n"
"Pressing the <label>Calculate</label> button will run the shortest path calculation using the optimization criterion selected in the <label>Criterion</label> combobox. The <label>Export</label> button allow you to export a calculated path to a new vector layer, and the button <label>Clear</label> will erase all fields and remove the points and calculated path from map canvas.</p>\n"
"\n"
"<h5>Note</h5>\n"
"<p>In order to bind the start and stop points of the route to the road network Road graph selects the nearest point or arc of the graph. In fact it can bind to any part of the road network. Nevertheless, their route and its characteristics do not take into account the distance from the starting point to the road network and of the road network to the stopping point.</p>\n"
"") );
  gContextHelpTexts.insert( "eVisDatabaseConnectionGui", QCoreApplication::translate( "context_help", "<h3>Database connection</h3>\n"
"") );
}
